@@ -8,6 +8,7 @@ from sqlalchemy.orm.collections import InstrumentedList
 from sqlalchemy.ext.declarative import declarative_base, DeclarativeMeta
 from sqlalchemy import Table, Column, Integer, String, Float, ForeignKey, UnicodeText, MetaData, Sequence, DateTime
 from sqlalchemy import and_, or_
+from sqlalchemy.sql.expression import literal
 
 from python_helper import log, Method, Function
 
@@ -25,6 +26,7 @@ Float = Float
 
 exists = exists
 select = select
+literal = literal
 
 relationship = relationship
 
@@ -94,13 +96,15 @@ class SqlAlchemyProxy:
     KW_MAIN_URL = 'main-url'
 
     KW_URL = 'url'
-    KW_REPOSITORY = 'repository'
+    KW_DATABASE = 'database'
     KW_REPOSITORY_DIALECT = 'dialect'
     KW_REPOSITORY_USER = 'user'
     KW_REPOSITORY_PASSWORD = 'password'
     KW_REPOSITORY_HOST = 'host'
     KW_REPOSITORY_PORT = 'port'
-    KW_REPOSITORY_DATABASE = 'database'
+    KW_REPOSITORY_SCHEMA = 'schema'
+    KW_REPOSITORY_URL = 'url'
+    KW_REPOSITORY_SETTINGS = 'settings'
 
     DATABASE_URL_ENIRONMENT_KEY = 'DATABASE_URL'
     DEFAULT_LOCAL_STORAGE_NAME = 'LocalStorage'
@@ -111,22 +115,22 @@ class SqlAlchemyProxy:
             model,
             globals,
             echo = False,
-            checkSameThread = False
+            connectArgs = None
         ):
 
         self.globals = globals
         self.sqlalchemy = sqlalchemy
-        dialect = self.globals.getSetting(f'{self.KW_API}{c.DOT}{self.KW_REPOSITORY}{c.DOT}{self.KW_REPOSITORY_DIALECT}')
-        self.engine = self.getNewEngine(dialect, echo, checkSameThread)
+        dialect = self.globals.getSetting(f'{self.KW_API}{c.DOT}{self.KW_DATABASE}{c.DOT}{self.KW_REPOSITORY_DIALECT}')
+        self.engine = self.getNewEngine(dialect, echo, connectArgs)
         self.session = scoped_session(sessionmaker(self.engine)) ###- sessionmaker(bind=self.engine)()
         self.model = model
         self.model.metadata.bind = self.engine
 
         self.run()
 
-    def getNewEngine(self, dialect, echo, checkSameThread) :
+    def getNewEngine(self, dialect, echo, connectArgs) :
         url = self.getUrl(dialect)
-        connectArgs = self.getConnectArgs(dialect, checkSameThread)
+        connectArgs = self.getConnectArgs(connectArgs)
         engine = None
         try :
             engine = create_engine(url, echo=echo, connect_args=connectArgs)
@@ -136,48 +140,57 @@ class SqlAlchemyProxy:
         return engine
 
     def getUrl(self, dialect) :
-        url = EnvironmentHelper.get(self.DATABASE_URL_ENIRONMENT_KEY)
+        log.log(self.getUrl, 'Loading repository configuration')
+        url = self.globals.getSetting(f'{self.KW_API}{c.DOT}{self.KW_DATABASE}{c.DOT}{self.KW_REPOSITORY_URL}')
         if isNeitherNoneNorBlank(url) :
+            log.log(self.getUrl, f'Prioritisig repository url in "{self.DATABASE_URL_ENIRONMENT_KEY}" environment variable')
             user = None
             password = None
             host = None
             port = None
-            name = None
+            schema = None
         else :
+            log.log(self.getUrl, 'Prioritisig repository yamel configuration')
             url = c.NOTHING
-            user = self.globals.getSetting(f'{self.KW_API}{c.DOT}{self.KW_REPOSITORY}{c.DOT}{self.KW_REPOSITORY_USER}')
-            password = self.globals.getSetting(f'{self.KW_API}{c.DOT}{self.KW_REPOSITORY}{c.DOT}{self.KW_REPOSITORY_PASSWORD}')
-            host = self.globals.getSetting(f'{self.KW_API}{c.DOT}{self.KW_REPOSITORY}{c.DOT}{self.KW_REPOSITORY_HOST}')
-            port = self.globals.getSetting(f'{self.KW_API}{c.DOT}{self.KW_REPOSITORY}{c.DOT}{self.KW_REPOSITORY_PORT}')
-            name = self.globals.getSetting(f'{self.KW_API}{c.DOT}{self.KW_REPOSITORY}{c.DOT}{self.KW_REPOSITORY_DATABASE}')
+            user = self.globals.getSetting(f'{self.KW_API}{c.DOT}{self.KW_DATABASE}{c.DOT}{self.KW_REPOSITORY_USER}')
+            password = self.globals.getSetting(f'{self.KW_API}{c.DOT}{self.KW_DATABASE}{c.DOT}{self.KW_REPOSITORY_PASSWORD}')
+            host = self.globals.getSetting(f'{self.KW_API}{c.DOT}{self.KW_DATABASE}{c.DOT}{self.KW_REPOSITORY_HOST}')
+            port = self.globals.getSetting(f'{self.KW_API}{c.DOT}{self.KW_DATABASE}{c.DOT}{self.KW_REPOSITORY_PORT}')
+            schema = self.globals.getSetting(f'{self.KW_API}{c.DOT}{self.KW_DATABASE}{c.DOT}{self.KW_REPOSITORY_SCHEMA}')
             if isNeitherNoneNorBlank(user) and isNeitherNoneNorBlank(password) :
                 url += f'{user}{c.COLON}{password}'
             if isNeitherNoneNorBlank(host) and isNeitherNoneNorBlank(port) :
                 url += f'{c.ARROBA}{host}{c.COLON}{port}'
             url += c.SLASH
-            name = f'{name}{c.DOT}{self.EXTENSION}' if isNeitherNoneNorBlank(name) else f'{self.DEFAULT_LOCAL_STORAGE_NAME if ObjectHelper.isNone(self.globals.apiName) else self.globals.apiName}{c.DOT}{self.EXTENSION}'
+            schema = f'{schema}{c.DOT}{self.EXTENSION}' if isNeitherNoneNorBlank(schema) else f'{self.DEFAULT_LOCAL_STORAGE_NAME if ObjectHelper.isNone(self.globals.apiName) else self.globals.apiName}{c.DOT}{self.EXTENSION}'
             if not isNeitherNoneNorBlank(dialect) :
                 dialect = self.DEFAULT_DIALECT
-            url = f'{dialect}{c.COLON}{c.DOUBLE_SLASH}{url}{name}'
+            url = f'{dialect}{c.COLON}{c.DOUBLE_SLASH}{url}{schema}'
         if SettingHelper.activeEnvironmentIsLocal() :
-            log.prettyPython(self.getUrl, 'Database coniguations', {
+            log.prettyPython(self.getUrl, 'Repository configuations', {**self.globals.getSetting(f'{self.KW_API}{c.DOT}{self.KW_DATABASE}'), **{
                 'dialect' : dialect,
                 'user' : user,
                 'password' : password,
                 'host' : host,
                 'port' : port,
-                'name' : name,
+                'schema' : schema,
                 'url' : url
-            }, logLevel = log.SETTING)
+            }}, logLevel = log.SETTING)
+        return url
+        if SettingHelper.activeEnvironmentIsLocal() :
+            log.prettyPython(self.getUrl, 'Database coniguations', self.globals.getSetting(f'{self.KW_API}{c.DOT}{self.KW_DATABASE}'), logLevel = log.SETTING)
         return url
 
-    def getConnectArgs(self, url, checkSameThread) :
-        if isNeitherNoneNorBlank(url) and url.startswith(self.DEFAULT_DIALECT) :
-            return {
-                'check_same_thread' : checkSameThread
-            }
+    def getConnectArgs(self, connectArgs) :
+        if ObjectHelper.isNone(connectArgs) :
+            connectArgs = self.globals.getSetting(f'{self.KW_API}{c.DOT}{self.KW_DATABASE}{c.DOT}{self.KW_REPOSITORY_SETTINGS}')
+            finalConnectArgs = {}
+            if ObjectHelper.isDictionary(connectArgs) :
+                for key, value in connectArgs.items() :
+                    finalConnectArgs[key] = value
+            return finalConnectArgs
         else :
-            return {}
+            return connectArgs
 
     @Method
     def run(self):
@@ -188,68 +201,68 @@ class SqlAlchemyProxy:
         self.session.commit()
 
     @Method
-    def save(self,instance):
+    def save(self, instance):
         self.session.add(instance)
 
     @Method
-    def saveNew(self,*args):
+    def saveNew(self, *args):
         model = args[-1]
         return self.save(model(*args[:-1]))
 
     @Method
-    def saveAndCommit(self,instance):
+    def saveAndCommit(self, instance):
         self.save(instance)
         self.session.commit()
         return instance
 
     @Method
-    def saveNewAndCommit(self,*args):
+    def saveNewAndCommit(self, *args):
         model = args[-1]
         return self.saveAndCommit(model(*args[:-1]))
 
     @Method
-    def saveAll(self,instanceList):
+    def saveAll(self, instanceList):
         self.session.add_all(instanceList)
         return instanceList
 
     @Method
-    def saveAllAndCommit(self,instanceList):
+    def saveAllAndCommit(self, instanceList):
         self.saveAll(instanceList)
         self.session.commit()
         return instanceList
 
     @Method
-    def findAll(self,model):
+    def findAll(self, model):
         objectList = self.session.query(model).all()
         return objectList
 
     @Method
-    def findAllAndCommit(self,model):
+    def findAllAndCommit(self, model):
         objectList = self.findAll(model)
         self.session.commit()
         return objectList
 
     @Method
-    def findByIdAndCommit(self,id,model):
+    def findByIdAndCommit(self, id, model):
         object = self.session.query(model).filter(model.id == id).first()
         self.session.commit()
         return object
 
     @Method
-    def existsByIdAndCommit(self,id,model):
+    def existsByIdAndCommit(self, id, model):
         # ret = Session.query(exists().where(and_(Someobject.field1 == value1, Someobject.field2 == value2)))
         objectExists = self.session.query(exists().where(model.id == id)).one()[0]
         self.session.commit()
         return objectExists
 
     @Method
-    def findByKeyAndCommit(self,key,model):
+    def findByKeyAndCommit(self, key, model):
         object = self.session.query(model).filter(model.key == key).first()
         self.session.commit()
         return object
 
     @Method
-    def existsByKeyAndCommit(self,key,model):
+    def existsByKeyAndCommit(self, key, model):
         objectExists = self.session.query(exists().where(model.key == key)).one()[0]
         self.session.commit()
         return objectExists
@@ -261,7 +274,13 @@ class SqlAlchemyProxy:
         return object
 
     @Method
-    def findAllByQueryAndCommit(self,query,model):
+    def existsByQueryAndCommit(self, query, model) :
+        exists = self.session.query(literal(True)).filter(self.session.query(model).filter_by(**query).exists()).scalar()
+        self.session.commit()
+        return exists
+
+    @Method
+    def findAllByQueryAndCommit(self, query, model):
         objectList = []
         if query :
             objectList = self.session.query(model).filter_by(**query).all()
@@ -269,7 +288,7 @@ class SqlAlchemyProxy:
         return objectList
 
     @Method
-    def deleteByKeyAndCommit(self,key,model):
+    def deleteByKeyAndCommit(self, key, model):
         if self.session.query(exists().where(model.key == key)).one()[0] :
             object = self.session.query(model).filter(model.key == key).first()
             self.session.delete(object)
